@@ -37,6 +37,38 @@ from .dvs.model_convtiny_handy import convtiny_dvs_handy
 from .dvs.model_convtiny_gesture import convtiny_dvs_gesture
 from .cwru.model_convtiny import convtiny_cwru
 from .mnist.model_gxnor import gxnor_mnist
+from .transformers.model_vit import vit_ti16, bc_vit_ti16
+from .transformers.model_deit import deit_ti16, bc_deit_ti16
+
+
+def add_transformers_args(parser, add_customs=False, default_norm='LN', default_act='GeLU'):
+    """ Add transformers commons arguments to the given parser.
+
+    Args:
+        parser (argparse.ArgumentParser): parser to add args to
+        add_customs (bool, optional): indicates if normalization, softmax and activation can be
+            customized. Defaults to False.
+        default_norm (str, optional): when add_customs=True, default value for normalization.
+            Defaults to 'LN'.
+        default_act (str, optional): when add_customs=True, default value for activation. Defaults
+            to 'GeLU'.
+    """
+    parser.add_argument("-c", "--classes", type=int, default=1000, help="The number of classes")
+    parser.add_argument("-bw", "--base_weights", type=str, default=None,
+                        help="Optional keras .h5 weights to load in the model.")
+    if add_customs:
+        norms = ['LN', 'GN1', 'BN', 'LMN']
+        parser.add_argument("--norm", type=str, choices=norms, default=default_norm,
+                            help="Replace normalization in model with a custom function, by \
+                                  default %(default)s")
+        sms = ['softmax', 'softmax2']
+        parser.add_argument("--softmax", type=str, choices=sms, default='softmax',
+                            help="Replace softmax operation in model with custom function, by \
+                                  default %(default)s")
+        acts = ['GeLU', 'ReLU8', 'swish']
+        parser.add_argument("--act", type=str, choices=acts, default=default_act,
+                            help="Replace activation function in model with custom function, \
+                                  by default %(default)s")
 
 
 def main():
@@ -185,6 +217,27 @@ def main():
                             transfer learning.")
     csp.add_parser("convtiny_cwru", help="A Convtiny CWRU model for akida")
     csp.add_parser("gxnor_mnist", help="A GXNOR MNIST model for akida")
+
+    vit_parser = csp.add_parser("vit_ti16", help="A ViT model")
+    add_transformers_args(vit_parser, add_customs=True)
+    vit_parser.add_argument("-i", "--image_size", type=int, default=224,
+                            choices=[224, 384], help="The square input image size")
+
+    bc_vit_parser = csp.add_parser("bc_vit_ti16", help="A ViT, brainchip customized model")
+    add_transformers_args(bc_vit_parser)
+
+    deit_parser = csp.add_parser("deit_ti16", help="A DeiT model")
+    add_transformers_args(deit_parser, add_customs=True)
+    deit_parser.add_argument("-i", "--image_size", type=int, default=224,
+                             choices=[160, 224], help="The square input image size")
+    deit_parser.add_argument("-d", "--dist", action="store_true",
+                             help="Use the model with a distilled token")
+
+    bc_deit_parser = csp.add_parser("bc_deit_ti16", help="A DeiT, brainchip customized model")
+    add_transformers_args(bc_deit_parser)
+    bc_deit_parser.add_argument("-d", "--dist", action="store_true",
+                                help="Use the model with a distilled token")
+
     args = parser.parse_args()
     if args.action == "create":
         # Instantiate the wished model
@@ -227,6 +280,22 @@ def main():
             model = convtiny_cwru()
         elif args.model == "gxnor_mnist":
             model = gxnor_mnist()
+        else:
+            # Handle transformer models
+            if args.model == "vit_ti16":
+                input_shape = (args.image_size, args.image_size, 3)
+                model = vit_ti16(input_shape=input_shape, classes=args.classes)
+            elif args.model == "bc_vit_ti16":
+                model = bc_vit_ti16(classes=args.classes)
+            elif args.model == "deit_ti16":
+                input_shape = (args.image_size, args.image_size, 3)
+                model = deit_ti16(input_shape=input_shape,
+                                  classes=args.classes, distilled=args.dist)
+            elif args.model == "bc_deit_ti16":
+                model = bc_deit_ti16(classes=args.classes, distilled=args.dist)
+            # Load provided weights
+            if args.base_weights is not None:
+                model.load_weights(args.base_weights, by_name=True)
         # No need for default behaviour as the command-line parser only accepts
         # valid model types
         model_path = args.save_model
