@@ -28,8 +28,8 @@ Cyclical learning rate policy module.
 Originated from https://github.com/bckenstler/CLR.
 """
 
-from tensorflow.keras import backend as K
-from tensorflow.keras.callbacks import Callback
+from keras import backend as K
+from keras.callbacks import Callback
 
 import numpy as np
 
