@@ -34,8 +34,6 @@ import glob
 
 import numpy as np
 
-from dv import AedatFile
-
 # Global definition for AEDATv2
 EVENT_LENGTH = 8  # 1 event takes 8 bytes (64 bits)
 READ_MODE = '>II'  # struct.unpack(), 4 bytes + 4 bytes
@@ -232,19 +230,25 @@ def _read_aedat_v4(datafile):
     ev_times = []
     events = []
 
-    with AedatFile(datafile) as f:
-        np_events = np.array([],
-                             dtype=[('timestamp', '<i8'), ('x', '<i2'),
-                                    ('y', '<i2'), ('polarity', 'i1'),
-                                    ('_p1', '<i2'), ('_p2', 'i1')])
-        for packet in f['events'].numpy():
-            np_events = np.hstack([np_events, packet])
+    try:
+        from dv import AedatFile
 
-        # access information of all events by type
-        timestamps, x, y, polarities = np_events['timestamp'], np_events[
-            'x'], np_events['y'], np_events['polarity']
-        ev_times = timestamps.tolist()
-        events = np.column_stack((x, y, polarities)).tolist()
+        with AedatFile(datafile) as f:
+            np_events = np.array([],
+                                 dtype=[('timestamp', '<i8'), ('x', '<i2'),
+                                        ('y', '<i2'), ('polarity', 'i1'),
+                                        ('_p1', '<i2'), ('_p2', 'i1')])
+            for packet in f['events'].numpy():
+                np_events = np.hstack([np_events, packet])
+
+            # access information of all events by type
+            timestamps, x, y, polarities = np_events['timestamp'], np_events[
+                'x'], np_events['y'], np_events['polarity']
+            ev_times = timestamps.tolist()
+            events = np.column_stack((x, y, polarities)).tolist()
+    except Exception as e:
+        raise ImportError(
+            f"dv package is required to read aedat v4 files: {e}") from e
 
     return events, ev_times
 
