@@ -27,21 +27,48 @@ import numpy as np
 import tensorflow as tf
 import tensorflow_datasets as tfds
 
-from keras.callbacks import (TensorBoard, ModelCheckpoint,
-                             LearningRateScheduler)
+import keras
+from keras.callbacks import TensorBoard, ModelCheckpoint, LearningRateScheduler, Callback
 from keras import Sequential
 from keras.optimizer_v2.gradient_descent import SGD
-from keras.layers import Input
+from keras.optimizer_v2.adam import Adam
+from tensorflow_addons.optimizers import LAMB
+from keras.layers import Input, Activation
 from keras.models import clone_model
 
 import akida
 from cnn2snn import load_quantized_model, convert
+from quantizeml.layers import AddPositionEmbs, ClassToken
+from quantizeml.models import load_model, set_calibrate
 
-from .preprocessing import preprocess_image
-from ..training import get_training_parser, freeze_model_before
+from .preprocessing import preprocess_image, DATA_AUGMENTATION
+from ..training import get_training_parser, freeze_model_before, print_history_stats
+from ..transformers.model_vit import apply_embedding_weights
 
 
-def get_imagenet_dataset(data_path, training, image_size, batch_size):
+class EarlyStoppingCalibration(Callback):
+    """A callback that limits the number of epochs to calibrate the model,
+        increasing the speed of the training.
+
+    Args:
+        trigger_epoch (int, optional): epoch at which to end the calibration.
+            Defaults to calibrate until the end of the training.
+
+    Notes:
+        If the :attr:`trigger_epoch` is set to 0, the model will never be calibrated
+    """
+
+    def __init__(self, trigger_epoch=-1):
+        self.trigger_epoch = trigger_epoch
+
+    def on_epoch_begin(self, epoch, logs=None):
+        calibrate = epoch < self.trigger_epoch or self.trigger_epoch < 0
+        set_calibrate(self.model, calibrate)
+        if logs is not None:
+            logs["calibrating"] = calibrate
+
+
+def get_imagenet_dataset(data_path, training, image_size, batch_size, data_aug=True, one_hot=False):
     """ Loads ImageNet 2012 dataset and builds a tf.dataset out of it.
 
     Args:
@@ -49,6 +76,8 @@ def get_imagenet_dataset(data_path, training, image_size, batch_size):
         training (bool): True to retrieve training data, False for validation
         image_size (int): desired image size
         batch_size (int): the batch size
+        data_aug (bool, optional): True to apply data augmentation (only train). Defaults to True.
+        one_hot (bool, optional): whether to one hot labels or not. Defaults to False.
 
     Returns:
         tf.dataset, int: the requested dataset (train or validation) and the
@@ -77,8 +106,14 @@ def get_imagenet_dataset(data_path, training, image_size, batch_size):
     if training:
         dataset = dataset.shuffle(10000, reshuffle_each_iteration=True).repeat()
 
+    data_aug = DATA_AUGMENTATION if data_aug else None
     dataset = dataset.map(lambda image, label: (preprocess_image(
-        image, image_size, training), label))
+        image, image_size, training, data_aug), label))
+
+    # One hot encode labels if requested
+    if one_hot:
+        num_classes = infos.features["label"].num_classes
+        dataset = dataset.map(lambda image, label: (image, tf.one_hot(label, num_classes)))
 
     dataset = dataset.batch(batch_size)
     dataset = dataset.prefetch(1)
@@ -91,15 +126,39 @@ def get_imagenet_dataset(data_path, training, image_size, batch_size):
     return dataset, infos.splits[split].num_examples / batch_size
 
 
-def compile_model(model):
+def compile_model(model, optimizer="SGD"):
     """ Compiles the model.
 
     Args:
         model (keras.Model): the model to compile
+        optimizer (str, optional): the optimizer to use. Defaults to "SGD".
+
+    Returns:
+        bool: True if labels should be one-hot encoded, False if not.
     """
-    model.compile(optimizer=SGD(momentum=0.9, nesterov=False),
-                  loss='sparse_categorical_crossentropy',
-                  metrics=['accuracy', 'sparse_top_k_categorical_accuracy'])
+    def _get_optim(optim_str):
+        optim_str_low = optim_str.lower()
+        if optim_str_low == "sgd":
+            return SGD(momentum=0.9)
+        elif optim_str_low == "adam":
+            return Adam(epsilon=1e-8)
+        elif optim_str_low == "lamb":
+            return LAMB(epsilon=1e-8, weight_decay_rate=2e-2)
+        else:
+            raise ValueError(f"Unknown optimizer {optim_str}. "
+                             "Please choose one of these options: {SGD, ADAM, LAMB}")
+
+    # Preserve legacy behavior where models have a softmax activation at the end
+    if isinstance(model.layers[-2], Activation):
+        model.compile(optimizer=_get_optim(optimizer),
+                      loss='sparse_categorical_crossentropy',
+                      metrics=['accuracy', 'sparse_top_k_categorical_accuracy'])
+        return False
+
+    model.compile(optimizer=_get_optim(optimizer),
+                  loss=keras.losses.CategoricalCrossentropy(from_logits=True),
+                  metrics=['accuracy', 'top_k_categorical_accuracy'])
+    return True
 
 
 def evaluate(model, val_dataset, batch_size, num_samples, val_steps):
@@ -148,7 +207,7 @@ def evaluate(model, val_dataset, batch_size, num_samples, val_steps):
         print(history)
 
 
-def rescale(base_model, input_size):
+def rescale_legacy(base_model, input_size):
     """ Rescales the model by changing its input size.
 
     Args:
@@ -176,6 +235,72 @@ def rescale(base_model, input_size):
     return clone
 
 
+def rescale_vit(base_model, input_size):
+    """ Rescales the model by changing its input size in three steps:
+        1. Change the input layer, given the new input size values
+        2. Reconstruct model from config
+        3. Update weights, taking into account interpolation on PosEmbed layer
+
+    Args:
+        base_model (keras.Model): the model to rescale
+        input_size (int): desired model input size
+
+    Returns:
+        keras.Model: the rescaled model
+    """
+    # 1. Create the desired input layer:
+    input_shape = (None, input_size, input_size, base_model.input.shape[-1])
+    num_tokens = sum(isinstance(ly, ClassToken) for ly in base_model.layers)
+
+    # 2. Clone the model by modification of dict_config:
+    # In models based on vision transformers, PositionEmbeding change its amount of parameters
+    # given an input size. This is why we need to reconstruct the model.
+    x_patch, y_patch = None, None
+    clone_config = base_model.get_config()
+
+    for layer_config in clone_config["layers"]:
+        # 2.1. Change input size
+        if layer_config["class_name"] == "InputLayer":
+            layer_config["config"]["batch_input_shape"] = input_shape
+        # 2.2. Recover total of patches in both directions
+        if "Conv2D" in layer_config["class_name"]:
+            x_patch = input_size // layer_config["config"]["kernel_size"][0]
+            y_patch = input_size // layer_config["config"]["kernel_size"][1]
+        # 2.3. Change values in reshape process
+        elif layer_config["class_name"] == "Reshape":
+            layer_config["config"]["target_shape"] = list(layer_config["config"]["target_shape"])
+            layer_config["config"]["target_shape"][0] = x_patch * y_patch
+
+    # 2.5. Recompile model with new configuration
+    clone = base_model.from_config(clone_config)
+
+    # 3. Update weights:
+    # Get weights from based model, and tranfer them into clone model,
+    for base_layer, clone_layer in zip(base_model.layers, clone.layers):
+        base_weights = base_layer.get_weights()
+        if isinstance(base_layer, AddPositionEmbs):
+            apply_embedding_weights(clone_layer, base_weights, x_patch, y_patch, num_tokens)
+        else:
+            clone_layer.set_weights(base_weights)
+    return clone
+
+
+def rescale(base_model, input_size):
+    """ Rescale the model by architecture (if there is a vision transformer model or not)
+
+    Args:
+        base_model (keras.Model): the model to rescale
+        input_size (int): desired model input size
+
+    Returns:
+        keras.Model: the rescaled model
+    """
+    is_vit = any(isinstance(layer, AddPositionEmbs) for layer in base_model.layers)
+    if is_vit:
+        return rescale_vit(base_model, input_size)
+    return rescale_legacy(base_model, input_size)
+
+
 def create_log_dir(out_dir):
     """ Creates a directory to store Tensorflow logs.
 
@@ -207,7 +332,8 @@ def train(model,
           num_epochs,
           tune=False,
           learning_rate=1e-1,
-          initial_epoch=0):
+          initial_epoch=0,
+          lrs_cos=False):
     """ Trains the model
 
     Args:
@@ -219,38 +345,69 @@ def train(model,
         out_dir (str): parent directory for logs folder
         num_epochs (int): the number of epochs
         tune (bool, optional): enable tuning (lower learning rate). Defaults to
-         False.
+          False.
         learning_rate (float, optional): the learning rate. Defaults to 1e-1.
         initial_epoch (int, optional): epoch at which to start training.
-         Defaults to 0.
+          Defaults to 0.
+        lrs_cos (bool, optional): True to use cosine learning rate scheduler instead of the default
+          exponential rule. Defaults to False.
     """
     # 1. Define training callbacks
     callbacks = []
 
-    # 1.1 Learning rate scheduler
-    LR_START = learning_rate
-    LR_END = 1e-4
-    # number of epochs you first keep the learning rate constant
-    LR_EPOCH_CONSTANT = 10
-    # Modify default values for fine-tuning
-    if tune:
-        LR_START = 1e-4
-        LR_END = 1e-8
-        LR_EPOCH_CONSTANT = 2
+    if lrs_cos:
+        # 1.1 Learning rate scheduler, modified by tune procedure
+        LR_START = learning_rate
+        if tune:
+            LR_EPOCH_CONSTANT = 2  # number of epochs you first keep the learning rate constant
+            LR_MIN = 1e-8
+        else:
+            LR_EPOCH_CONSTANT = 5  # number of epochs you first keep the learning rate constant
+            LR_MIN = 1e-6
+        # Make sure to start with a learning rate higher than LR_MIN
+        LR_MIN = LR_MIN if LR_START > LR_MIN else 0.1*LR_START
 
-    if LR_EPOCH_CONSTANT >= num_epochs:
-        lr_decay = LR_END / LR_START
+        # If the number of epochs is too small, dont use WARMUP_LR
+        if num_epochs < LR_EPOCH_CONSTANT:
+            LR_EPOCH_CONSTANT = 0
+
+        # This function keeps the learning rate at LR_START for the first N epochs and decreases it
+        # following
+        # https://www.tensorflow.org/api_docs/python/tf/keras/optimizers/schedules/CosineDecay
+        def cos_lr_scheduler(epoch):
+            if epoch < LR_EPOCH_CONSTANT:
+                return LR_START
+            step = min(epoch, num_epochs) - LR_EPOCH_CONSTANT
+            cosine_decay = 0.5 * (1 + np.cos(np.pi * step / (num_epochs - LR_EPOCH_CONSTANT)))
+            decayed = (1 - LR_MIN) * cosine_decay + LR_MIN
+            return LR_START * decayed
+
+        callbacks.append(LearningRateScheduler(cos_lr_scheduler))
     else:
-        lr_decay = (LR_END / LR_START)**(1. / (num_epochs - LR_EPOCH_CONSTANT))
+        # 1.1 Learning rate scheduler
+        LR_START = learning_rate
+        LR_END = 1e-4
+        # number of epochs you first keep the learning rate constant
+        LR_EPOCH_CONSTANT = 10
+        # Modify default values for fine-tuning
+        if tune:
+            LR_START = 1e-4
+            LR_END = 1e-8
+            LR_EPOCH_CONSTANT = 2
 
-    # This function keeps the learning rate at LR_START for the first N epochs
-    # and decreases it exponentially after that.
-    def agg_lr_scheduler(epoch):
-        if epoch < LR_EPOCH_CONSTANT:
-            return LR_START
-        return LR_START * lr_decay**(epoch - (LR_EPOCH_CONSTANT - 1))
+        if LR_EPOCH_CONSTANT >= num_epochs:
+            lr_decay = LR_END / LR_START
+        else:
+            lr_decay = (LR_END / LR_START)**(1. / (num_epochs - LR_EPOCH_CONSTANT))
 
-    callbacks.append(LearningRateScheduler(agg_lr_scheduler))
+        # This function keeps the learning rate at LR_START for the first N epochs
+        # and decreases it exponentially after that.
+        def agg_lr_scheduler(epoch):
+            if epoch < LR_EPOCH_CONSTANT:
+                return LR_START
+            return LR_START * lr_decay**(epoch - (LR_EPOCH_CONSTANT - 1))
+
+        callbacks.append(LearningRateScheduler(agg_lr_scheduler))
 
     # 1.2 Model checkpoints (save best models after each epoch)
     log_dir = create_log_dir(out_dir)
@@ -273,6 +430,10 @@ def train(model,
                               profile_batch=0)
     callbacks.append(tensorboard)
 
+    # 1.4 Custom callbacks
+    # Callback to calibrate the model only on the first epoch
+    callbacks.append(EarlyStoppingCalibration(trigger_epoch=1))
+
     # 2. Train
     history = model.fit(train_dataset,
                         steps_per_epoch=train_steps,
@@ -281,7 +442,7 @@ def train(model,
                         validation_data=val_dataset,
                         validation_steps=val_steps,
                         initial_epoch=initial_epoch)
-    print(history.history)
+    print_history_stats(history)
 
 
 def main():
@@ -316,6 +477,13 @@ def main():
                               type=int,
                               default=0,
                               help="Epoch at which to start training.")
+    train_parser.add_argument("--optim", type=str, default="SGD",
+                              help="Optimizer to use. Defaults to %(default)s.")
+    train_parser.add_argument("--data_aug", action='store_true', help="Enables custom DA.")
+    train_parser.add_argument("--lrs_cos",
+                              action='store_true',
+                              help="Use cosine learning rate scheduler instead of the exponential \
+                                    default.")
 
     tune_parser = parsers[2]
     tune_parser.add_argument("-ie",
@@ -323,6 +491,22 @@ def main():
                              type=int,
                              default=0,
                              help="Epoch at which to start training.")
+    tune_parser.add_argument("-lr",
+                             "--learning_rate",
+                             type=float,
+                             default=6e-5,
+                             help="Learning rate start value.")
+    tune_parser.add_argument("--data_aug",
+                             action='store_true',
+                             help="Enables custom DA.")
+    tune_parser.add_argument("--optim",
+                             type=str,
+                             default="SGD",
+                             help="Optimizer to use. Defaults to %(default)s.")
+    tune_parser.add_argument("--lrs_cos",
+                             action='store_true',
+                             help="Use cosine learning rate scheduler instead of the exponential \
+                                   default.")
 
     eval_parser = parsers[3]
     eval_parser.add_argument("-ns",
@@ -349,26 +533,40 @@ def main():
     args = parsers[0].parse_args()
 
     # Load the source model
-    model = load_quantized_model(args.model)
+    try:
+        model = load_quantized_model(args.model)
+    except Exception:
+        try:
+            model = load_model(args.model)
+        except Exception as e:
+            raise e.__class__('Cannot load provided model.')
 
     # Freeze the model
     if "freeze_before" in args:
         freeze_model_before(model, args.freeze_before)
 
     # Compile model
-    compile_model(model)
+    one_hot = compile_model(model, optimizer=getattr(args, "optim", "SGD"))
 
     # Load validation data
-    im_size = model.input_shape[1]
-    val_ds, val_steps = get_imagenet_dataset(args.data_dir, False, im_size,
-                                             args.batch_size)
+    if args.action != 'rescale':
+        im_size = model.input_shape[1]
+        val_ds, val_steps = get_imagenet_dataset(args.data_dir, False, im_size,
+                                                 args.batch_size, data_aug=False, one_hot=one_hot)
 
     # Train model
     if args.action in ['train', 'tune']:
         # Load training data
         train_ds, train_steps = get_imagenet_dataset(args.data_dir, True,
-                                                     im_size, args.batch_size)
+                                                     im_size, args.batch_size,
+                                                     args.data_aug, one_hot=one_hot)
         tune = args.action == 'tune'
+
+        learning_rate = args.learning_rate
+        if args.lrs_cos:
+            # Tune learning rate following https://arxiv.org/pdf/2012.12877.pdf
+            learning_rate *= args.batch_size / 512
+
         train(model,
               train_ds,
               train_steps,
@@ -377,8 +575,9 @@ def main():
               args.out_dir,
               args.epochs,
               tune=tune,
-              learning_rate=args.learning_rate if not tune else 1e-1,
-              initial_epoch=args.initial_epoch)
+              learning_rate=learning_rate,
+              initial_epoch=args.initial_epoch,
+              lrs_cos=args.lrs_cos)
 
         # Save model in Keras format (h5)
         if args.savemodel:
