@@ -28,7 +28,7 @@ from cnn2snn import load_quantized_model, convert
 import numpy as np
 
 from ..distiller import Distiller, KLDistillationLoss
-from ..training import get_training_parser, compile_model, evaluate_model
+from ..training import get_training_parser, compile_model, evaluate_model, print_history_stats
 
 
 def get_data():
@@ -75,7 +75,7 @@ def train_model(model, teacher_model, x_train, y_train, batch_size, epochs):
 
     # Train the model
     history = distiller.fit(x_train, y_train, batch_size, epochs)
-    print(history.history)
+    print_history_stats(history)
 
 
 def train_model_classical(model, x_train, y_train, batch_size, epochs):
@@ -89,7 +89,7 @@ def train_model_classical(model, x_train, y_train, batch_size, epochs):
         epochs (int): the number of epochs
     """
     history = model.fit(x_train, y_train, batch_size, epochs)
-    print(history.history)
+    print_history_stats(history)
 
 
 def main():
