@@ -18,9 +18,8 @@
 PointNet++ model definition for ModelNet40 classification.
 """
 
-from tensorflow.keras import layers, Model
-from tensorflow.keras import regularizers
-from tensorflow.keras.utils import get_file
+from keras import layers, Model, regularizers
+from keras.utils.data_utils import get_file
 
 from cnn2snn import quantize, load_quantized_model
 
@@ -75,7 +74,7 @@ def pointnet_plus_modelnet40(selected_points=128,
             * '2' through '8' implements n-bit weights where n is from 2-8 bits.
 
     Returns:
-        tf.keras.Model: a quantized Keras model for PointNet++/ModelNet40.
+        keras.Model: a quantized Keras model for PointNet++/ModelNet40.
     """
     # Adapt input shape with preprocessing
     reshape_factor = get_reshape_factor(knn_points)
@@ -199,7 +198,7 @@ def pointnet_plus_modelnet40_pretrained():
     ModelNet40 dataset.
 
     Returns:
-        tf.keras.Model: a Keras Model instance.
+        keras.Model: a Keras Model instance.
     """
     model_name = 'pointnet_plus_modelnet40_iq8_wq4_aq4.h5'
     file_hash = '464f9cccc868c46eb4ba16b722a2e7da86678dd8f50ec6bd988b965614022fc9'
