@@ -156,12 +156,12 @@ def get_training_parser(batch_size,
 
     Args:
         batch_size (int): batch size default value
-        tune (bool, optional): either to add a tune action or not. Default to
+        tune (bool, optional): either to add a tune action or not. Defaults to
             False.
         freeze_before (bool, optional): either to add a "--freeze_before"
-            parameter in train and tune actions or not. Default to False.
+            parameter in train and tune actions or not. Defaults to False.
         global_batch_size (bool, optional): either to add batch size as a
-            global parser parameter or for train/tune actions only. Default to
+            global parser parameter or for train/tune actions only. Defaults to
             True.
         global_parser (argparse.ArgumentParser, optional): global parser with
             custom parameters. Defaults to None.
@@ -253,3 +253,19 @@ def get_training_parser(batch_size,
                              help="Converts to an Akida model and evaluate")
 
     return parser, train_parser, tune_parser, eval_parser, subparsers
+
+
+def print_history_stats(history):
+    """Prints a basic statistics of the training/eval history.
+
+    Args:
+        history (keras.callbacks.History): history of the training/eval process
+    """
+    print("\nHistory statistics:\n" + "-" * 80 + "\n")
+    max_len_metric = max(len(metric) for metric in history.history.keys())
+    clean_history = {key.ljust(max_len_metric): values for key, values in history.history.items()}
+    stats = [np.min, np.max, np.mean, np.std]
+    print("Parameter:".ljust(max_len_metric) + "\t" + "\t".join(f.__name__.ljust(6) for f in stats))
+    for key, values in clean_history.items():
+        values = "\t".join(f"{f(values):.4f}" for f in stats)
+        print(f"{key}\t{values}")
